from telliot_core.plugin.registry import PluginRegistry

# from telliot_feed_examples.feeds.btc_usd_feed import btc_usd_median_feed

registry = PluginRegistry()
# registry.register_feed(btc_usd_median_feed)

__version__ = "0.0.14"
